/*
 * MIT License
 *
 * Copyright (c) 2019 norangebit
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

package it.norangeb.algorithms.graph.operations

import it.norangeb.algorithms.datastructures.queue.priority.BinaryHeap
import it.norangeb.algorithms.datastructures.queue.priority.PriorityQueue
import it.norangeb.algorithms.datastructures.unionfind.QuickFind
import it.norangeb.algorithms.datastructures.unionfind.UnionFind
import it.norangeb.algorithms.graph.WeightUndirectedGraph
import it.norangeb.algorithms.graph.data.EdgeI

class Kruskal(graph: WeightUndirectedGraph) : MinimumSpanningTree {
    val mst = mutableListOf<EdgeI>()

    init {
        val minQueue = BinaryHeap.createMinPriorityQueue<EdgeI>()
        val unionFindGraph = QuickFind(graph.vertexNumber())

        graph.edges().forEach { minQueue.insert(it) }

        while (!minQueue.isEmpty() && mst.size < graph.vertexNumber() - 1)
            tryEdge(minQueue, unionFindGraph)
    }

    private fun tryEdge(heap: PriorityQueue<EdgeI>, unionFindGraph: UnionFind) {
        heap.pop().map {
            val from = it.either()
            val to = it.other(from)

            if (!unionFindGraph.connected(from, to)) {
                unionFindGraph.union(from, to)
                mst.add(it)
            }
        }
    }

    override fun edges(): Collection<EdgeI> = mst

    override fun weight(): Double = mst.map { it.weight() }.sum()
}